#!/bin/sh
#
# FW-Lite
# Copyright (C) 2003-2005 Truelite s.r.l
# by Simone Piccardi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program or from the site that you downloaded it
# from; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA  02111-1307   USA
#

MYNAME="$0"

# find default gateway interface 
if [ -x /sbin/ip ]; then
    IFACE=`/sbin/ip route | grep default | head -n1 | cut -d" " -f 5`
elif [ -x /sbin/route ]; then
    IFACE=`/sbin/route | grep default | head -n1 | tr -s " " | cut -d" " -f 8`
else 
    echo "Cannot find gateway interface"
fi

# find local IP
#if [ -x /sbin/ifconfig ]; then
#    LOCALIP=`/sbin/ifconfig | sed -n -r -e '/Link encap:Ether/,+1 p' | sed -r -e "/$IFACE/,+1 d" | grep inet | cut -d: -f2| cut -d" " -f1`
#else 
 #    echo "Cannot find local IP value"
#fi
## find local net
#if [ -x /sbin/ip ]; then
#    LOCALNET=`/sbin/ip route | grep $LOCALIP | cut -d" " -f 1`
#else 
#    echo "Cannot find local net value"
#fi

if [ -x /sbin/ifconfig ]; then
    LOCALIP=`/sbin/ifconfig | sed -n -r -e "/$IFACE/,+1 d" | grep inet | cut -d: -f2| cut -d" " -f1`
else 
    echo "Cannot find external IP value"
fi

LOCALNET=<subnet>/<netmask>
SERVERIP=<server-ip>

echo "Masquerading via $IFACE. Gateway is $SERVERIP, network is $LOCALNET"


case $1 in
  start|restart|reload|force-reload)
    # Clean all
    iptables -F
    iptables -t mangle -F
    iptables -t nat -F
    iptables -X allowed
    iptables -X services
    iptables -X outbound

    #
    # Turn on/off some kernel features
    # 
    echo 1 > /proc/sys/net/ipv4/ip_forward
    echo 1 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts
    echo 0 > /proc/sys/net/ipv4/conf/all/accept_source_route
    echo 0 > /proc/sys/net/ipv4/conf/all/accept_redirects
    #echo 1 > /proc/sys/net/ipv4/icmp_ignore_bogus_error_responses

    #
    # POSTROUTING, to NAT
    #
    iptables -t nat -A POSTROUTING -o $IFACE -j MASQUERADE
    # DA CORREGGERE, DOPO AVER CONFIGURATO DANSGUARDIAN
    iptables -t nat -A PREROUTING -s  $LOCALNET -d ! $SERVERIP -p tcp --dport 80 -j REDIRECT --to-ports 8080
    iptables -t nat -A PREROUTING -d ! $SERVERIP -p tcp --dport 53 -j REDIRECT --to-ports 53
    iptables -t nat -A PREROUTING -d ! $SERVERIP -p udp --dport 53 -j REDIRECT --to-ports 53


    # Chain allowed, for generic allowed traffic, for both INPUT and FORWARD
    iptables -N allowed
    iptables -A allowed -m state --state ESTABLISHED,RELATED -j ACCEPT
    iptables -A allowed -p icmp --icmp-type destination-unreachable -j ACCEPT
    iptables -A allowed -p icmp --icmp-type time-exceeded -j ACCEPT
    iptables -A allowed -p icmp --icmp-type echo-request -j ACCEPT
    iptables -A allowed -p icmp --icmp-type fragmentation-needed -j ACCEPT

    # Chain services, for services on the router/firewall
    # per ora ssh, http, https e proxy
    iptables -N services
    iptables -A services -p tcp --dport   22 -j ACCEPT
    iptables -A services -p tcp --dport   80 -j ACCEPT
    iptables -A services -p tcp --dport  443 -j ACCEPT
    iptables -A services -p tcp --dport 3128 -j ACCEPT

    # Chain outbound, for outbound connections
    # when empy nothing is passing, limit as you prefer
    iptables -N outbound
    # iptables -A outbound -m state --state NEW  -j ACCEPT # allow all
    iptables -A outbound -p tcp --dport 443 -m state --state NEW  -j ACCEPT # allow https
    iptables -A outbound -p udp --dport 123 -m state --state NEW  -j ACCEPT # allow NTPDATE query
    iptables -A outbound -p tcp --dport 2628 -m state --state NEW  -j ACCEPT # allow dict query


    #
    # INPUT
    #
    # localhost traffic is allowed
    iptables -A INPUT -i lo -s 127.0.0.1 -d 127.0.0.1 -j ACCEPT
    iptables -A INPUT -s $LOCALNET -m state --state NEW -j ACCEPT
    iptables -A INPUT -j allowed
    iptables -A INPUT -m state --state NEW -j services
    iptables -A INPUT -j DROP

    #
    # OUTPUT
    #
    # localhost always allowed
    iptables -A OUTPUT -o lo -s 127.0.0.1 -d 127.0.0.1 -j ACCEPT
    # outgoing from local allowed
    iptables -A OUTPUT -m state --state NEW,ESTABLISHED,RELATED -j ACCEPT
    iptables -A OUTPUT -j DROP

    #
    # FORWARD
    #
    iptables -A FORWARD -j allowed
    iptables -A FORWARD -o $IFACE -j outbound
    iptables -A FORWARD -j DROP
    ;;

  stop) 
    # Clean all
    iptables -F
    iptables -t mangle -F
    iptables -t nat -F
    iptables -X allowed
    iptables -X services
    iptables -X outbound

    #
    # Revert on/off some kernel features
    # 
    echo 0 > /proc/sys/net/ipv4/ip_forward
    echo 0 > /proc/sys/net/ipv4/icmp_echo_ignore_broadcasts
    echo 1 > /proc/sys/net/ipv4/conf/all/accept_source_route
    echo 1 > /proc/sys/net/ipv4/conf/all/accept_redirects
    exit 0
    ;;

  *)
    echo "Usage: $MYNAME {stop|start|restart|reload|force-reload}" >&2
    exit 3
    ;;
esac

